import csv
import re
import requests
import getpass
import datetime
import os
import ast
import smtplib
from email.message import EmailMessage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import urllib3
from urllib.parse import urlparse
import tldextract
from time import sleep

# API END POINTS: Please note that we can add more endpoints on demand
CLIENT_ID = 'client-api'
CLIENT_SECRET = 'comfy-litigate-embargo-forelimb'
AUTH_URL = 'https://app.leak.center/uaa/oauth/token'
API_ENDPOINTS = {
    'domain': "https://app.leak.center/svc-saas/domain/search",
    'accounts': "https://app.leak.center/svc-saas/account/search",
    'leaks': "https://app.leak.center/svc-saas/leak/search",
    'certificates': "https://app.leak.center/svc-saas/certificate/search",
    'botnets': "https://app.leak.center/svc-saas/bot/record/search",
    'ransomware': "https://app.leak.center/svc-saas/ransomware/search",
    'cc': "https://app.leak.center/svc-saas/cc/search"
}


DOMAIN_REGEX = r"^(?:[a-zA-Z0-9](?:[a-zA-Z0-9-]{0,61}[a-zA-Z0-9])?\.)+[a-zA-Z]{2,}$"
SECOND_LEVEL_DOMAIN_REGEX = r"^[a-zA-Z0-9-]{1,63}$"
IP_RANGE_REGEX = r"^(?:\d{1,3}\.){3}\d{1,3}(?:/\d{1,2})?$"
ASSET_OPTIONS = {
    '1': 'Domain',
    '2': 'Second-level domain',
    '3': 'IP range'
}
QUERY_OPTIONS = {
    '0': 'domain',
    '1': 'accounts',
    '2': 'leaks',
    '3': 'certificates',
    '4': 'botnets',
    '5': 'cc',
    '6': 'ransomware',
    '7': 'all',  # '7' is the default option, so we can use it to query all
}
RESULTS_FOUND_PER_QUERY = {
    'accounts': 0,
    'leaks': 0,
    'certificates': 0,
    'botnets': 0,
    'cc': 0,
    'ransomware': 0,
    'domain': 0
}

# Function to validate domain or IP range
def is_valid_domain_or_ip(asset_name, asset_option):
    if asset_option in ['1', '2']:  # Domain or Second-level domain
        return re.match(DOMAIN_REGEX, asset_name) or re.match(SECOND_LEVEL_DOMAIN_REGEX, asset_name)
    elif asset_option == '3':  # IP range
        return re.match(IP_RANGE_REGEX, asset_name)
    return False



# Function to get the date range from the user or CSV
def get_date_range():
    while True:
        date_input = input("Enter the date range in the format 'YYYY-MM-DD TO YYYY-MM-DD': ")
        # Validate date input
        try:
            start_date, end_date = date_input.split(' TO ')
            # Attempt to parse the dates to ensure they are in the correct format
            datetime.datetime.strptime(start_date, '%Y-%m-%d')
            datetime.datetime.strptime(end_date, '%Y-%m-%d')
            return f"createdAt:[{start_date} TO {end_date}]"
        except ValueError:
            print("Invalid date format. Please try again using the format 'YYYY-MM-DD TO YYYY-MM-DD'.")

def get_input_from_csv():
    filename = input("Enter the path to your CSV file: ")
    clients = []
    with open(filename, mode='r', newline='', encoding='utf-8-sig') as csvfile:
        # Using DictReader to read the CSV as a dictionary
        reader = csv.DictReader(csvfile)

        # Parsing 'date_range' from the CSV instead of 'start_date' and 'end_date'
        for row in reader:
            row = dict(row)
            print(row)
            try:
                date_range = row['date_range']
                start_date, end_date = date_range.split(' TO ')
                datetime.datetime.strptime(start_date, '%Y-%m-%d')

                if "*" not in end_date:
                    datetime.datetime.strptime(end_date, '%Y-%m-%d')
                else:
                    end_date = datetime.datetime.today().strftime('%Y-%m-%d')
                row['date_range'] = f"createdAt:[{start_date} TO {end_date}]"
                row['query_date_from'] = start_date
                row['query_date_to'] = end_date
            except ValueError:
                raise Exception("Invalid field `date_range` in CSV. Dates must be in 'YYYY-MM-DD TO YYYY-MM-DD' format.")

            # If 'asset_name' is not a list, convert it to a list forcefully
            # January 29: removed 'asset_name' from the CSV, added 'asset_name_domain' and 'asset_name_other' instead
            # try:
            #     assets_list = ast.literal_eval(row['asset_name'])
            #     row['asset_name'] = assets_list
            # except:
            #     row['asset_name'] = [row['asset_name']]

            try:
                assets_list_domain = ast.literal_eval(row['asset_name_domain'])
                row['asset_name_domain'] = assets_list_domain
            except:
                row['asset_name_domain'] = [row['asset_name_domain']]

            try:
                assets_list_other = ast.literal_eval(row['asset_name_other'])
                row['asset_name_other'] = assets_list_other
            except:
                row['asset_name_other'] = [row['asset_name_other']]

            # If 'credit_card_owner' is not a list, convert it to a list forcefully
            try:
                cc_owners_list = ast.literal_eval(row['credit_card_owner'])
                row['credit_card_owner'] = cc_owners_list
            except:
                row['credit_card_owner'] = [row['credit_card_owner']]

            # If 'credit_card_number' is not a list, convert it to a list forcefully
            try:
                cc_numbers_list = ast.literal_eval(row['credit_card_number'])
                row['credit_card_number'] = cc_numbers_list
            except:
                row['credit_card_number'] = [row['credit_card_number']]

            clients.append(row)

    if clients:
        return clients
    else:
        raise Exception("No information found in the .csv file. Please check the README.md file")

# Function to manually get input data with validation
def get_manual_input():
    print("Please provide the following details:")

    # Validate asset option
    while True:
        asset_option = input("Asset type - Domain (1), 2nd-level domain (2), IP range (3): ")
        if asset_option in ASSET_OPTIONS:
            break
        else:
            print(f"Invalid input. Please choose a valid option: {', '.join(ASSET_OPTIONS.keys())}")

    # Validate query type
    while True:
        query_type = input("Query type - Accounts (1), Leaks (2), Certificates (3), Botnets (4), All (5): ")
        if query_type in QUERY_OPTIONS:
            break
        else:
            print(f"Invalid input. Please choose a valid option: {', '.join(QUERY_OPTIONS.keys())}")

    asset_name_input = input(f"Asset name ({ASSET_OPTIONS[asset_option]}): ")

    # Validate asset name based on asset option
    while not is_valid_domain_or_ip(asset_name_input, asset_option):
        print("Invalid asset name. Please enter a valid domain, 2nd-level domain, or IP range.")
        asset_name_input = input(f"Asset name ({ASSET_OPTIONS[asset_option]}): ")

    # Get the date range from the user
    date_range_input = get_date_range()

    return {
        'username': input("Username: "),
        'password': getpass.getpass("Password: "),
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET,
        'query_type': QUERY_OPTIONS[query_type],
        'asset_option': asset_option,
        'asset_name': asset_name_input,
        'client_name': input("Client name: "),
        'date_range': date_range_input,  # Include the date range in the returned data
        'alert': input("Alert? (0/1): "),
        'alert_email': input("Alert email: "),
        'sender_email': input("Sender email (will be used for notifications): "),
        'sender_password': getpass.getpass("Sender password: "),
        'company_name': input("Company name: "),
        'company_contact': input("Company contact: "),
        'smtp_server': input("SMTP server (make sure you're using encrypted connection): "),
        'smtp_port': input("SMTP port: "),
        'credit_card_number': input("Credit card number: "),
        'credit_card_owner': input("Credit card owner: "),
    }

# Function to authenticate and get token
def get_access_token(auth_data):
    response = requests.post(AUTH_URL, data={
        'grant_type': 'password',
        'username': auth_data['username'],
        'password': auth_data['password'],
        'client_id': auth_data['client_id'],
        'client_secret': auth_data['client_secret']
    })
    response.raise_for_status()
    return response.json()['access_token']

    
# Function to perform API query and loop through all pages
def perform_query(endpoint, token, query_type, asset_name, page=0, size=100):
    all_results = []
    try:
        headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json'
        }
        
        while True:  # Loop through pages
            params = {
                'query': asset_name,
                'page': page,
                'size': size
            }
            
            # Add 'sort' parameter conditionally for 'accounts'
            if query_type == 'accounts':
                params['sort'] = 'name,asc'

            elif query_type == 'leaks':
                params['length'] = 1000
            
            print(params)
            for attempt in range(2):
                response = requests.get(endpoint, headers=headers, params=params)
                print(response.status_code)
                if response.status_code != 200:
                    sleep(60)
                    continue
                else:
                    break
            data = response.json()
            print(data)
            
            all_results.extend(data['content'])  # Add current page's results to all_results

            # Break the loop if no content is found or we reach the last page
            if not data.get('content') or data.get('last', False):
                break
            
            page += 1  # Go to the next page

        return all_results
    except Exception as e:
        print(f"An error occurred while querying {query_type}: {e}")
        return None


def perform_query_leaks(endpoint, token, query, query_date_from, query_date_to, page=0, size=100):
    all_results = []
    has_query_limit_exceeded, last_record_timestamp = False, None
    try:
        headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json'
        }
        while True:  # Loop through pages
            params = {
                'query': f"{query} AND createdAt:[{query_date_from} TO {query_date_to}]",
                #'query': query,
                'page': page,
                'size': size,
                'length': 1000,
                'sort': 'createdAt,asc'
            }

            print(f"DO: {params}")
            for attempt in range(2):
                response = requests.get(endpoint, headers=headers, params=params)
                print(response.status_code)
                if response.status_code != 200:
                    sleep(60)
                    continue
                else:
                    break
            data = response.json()
            # print(data)
            print(f"OK: page {page}. dates:[{query_date_from} TO {query_date_to}]. totalPages {data['totalPages']}, totalElements {data['totalElements']}")

            all_results.extend(data['content'])  # Add current page's results to all_results

            page += 1  # Go to the next page

            # Break the loop if no content is found or we reach the last page
            if not data.get('content') or data.get('last', False):
                if data['totalElements'] >= 10000:
                    # keep on querying until we get all results, using the last record timestamp
                    # 'createdAt' => '2022-08-16 14:57:00' should be converted to '2022-08-16'
                    has_query_limit_exceeded = True
                    last_record_timestamp = data['content'][-1]['createdAt']
                    query_date_from = last_record_timestamp.split()[0]
                    # start from page 0 again
                    page = 0
                    print(f"OK: page {page}. has more than 10000 results. Updated daterange [{query_date_from} TO {query_date_to}].")
                    continue
                else:
                    break



        return all_results
    except Exception as e:
        print(f"An error occurred while querying leaks: {e}")
        return None



# Function to save results to CSV - please note that we can also integrate this in your existing ticketing system (Splunk, Jira, Slack etc)
def save_to_csv(client_name, query_type, results):
    # Check if there are any results to save
    if not results:
        print(f"No results to save for {query_type}.")
        return

    directory_cannot_be_created = False
    try:
        os.mkdir('output')
    except FileExistsError:
        pass
    except Exception as e:
        directory_cannot_be_created = True

    # Create the output directory if it doesn't exist
    output_dir = os.path.join('output', client_name)  # Use os.path.join for correct path handling
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    # Define the filename for the CSV
    filename = f'{query_type}.csv'  # The file name

    # Define the path for the CSV including the directory
    filepath = os.path.join(output_dir, filename)  # The full path for the file

    # If the directory cannot be created, save the file in the current directory
    if directory_cannot_be_created:
        filepath = filename

    # write each row to csv
    current_row = ""
    with open(filepath, 'w', newline='', encoding='utf-8', errors='surrogatepass') as csvfile:
        # don't use Dictrow
        writer = csv.writer(csvfile)
        writer.writerow(results[0].keys())
        for row in results:
            current_row = row
            cleaned_row = []
            for value in row:
                cleaned_value = str(row[value]).replace('\n', ' ').replace('\r', ' ').replace('\t', ' ')
                cleaned_row.append(cleaned_value)
            #cleaned_row = cleaned_row.encode('utf-8').decode('utf-8')
            writer.writerow(cleaned_row)
            # cleaned_row = [_.encode('utf-16','surrogatepass').decode('utf-16') for _ in cleaned_row if _]


    print(f"Results saved to {filepath}")
    return filepath

def send_email(alert_email, sender_email, sender_password, company_name, company_contact, alert_count, smtp_server, smtp_port, attachments):
    html = """
    <!DOCTYPE html>
    <html>
    <head>
        <title>{{COMPANY_NAME}} Security Alert</title>
        <style>
            body {
                font-family: 'Arial', sans-serif;
                margin: 0;
                padding: 0;
                background-color: #f4f4f4;
            }
            .email-container {
                max-width: 600px;
                margin: 20px auto;
                background: #ffffff;
                border: 1px solid #ddd;
                border-radius: 4px;
                overflow: hidden;
            }
            .email-header {
                background-color: #e9e9e9;
                padding: 10px 20px;
                text-align: center;
            }
            .email-body {
                padding: 20px;
            }
            .email-footer {
                background-color: #e9e9e9;
                color: #333;
                padding: 10px 20px;
                text-align: center;
                font-size: 0.8em;
            }
            .alert-table {
                width: 100%;
                border-collapse: collapse;
                margin-bottom: 20px;
            }
            .alert-table th, .alert-table td {
                text-align: left;
                padding: 8px;
                border: 1px solid #ddd;
                font-size: 0.9em;
            }
        </style>
    </head>
    <body>
        <div class="email-container">
            <div class="email-header">
               <h1>{{COMPANY_NAME}} </h1>
               <h2>DARK- & DEEP WEB ALERT NOTIFICATION</h2>
               
            </div>
            <div class="email-body">
                <h3>There are {{ALERT_COUNT}} new alert(s).</h3>
                <p>Read below for description and impact from each alert on your assets.</p>
                <table class="alert-table">
                    <!-- Table headers -->
                    <tr>
                        <th>Type of Alert</th>
                        <th>Count</th>
                    </tr>
                    <!-- Table rows for alerts -->
                    <tr>
                        <td>Certificate Alerts</td>
                        <td>{{SSL_ALERT_COUNT}}</td>
                    </tr>
                    <!-- Botnet Alerts -->
                    <tr>
                        <td>Botnet Alerts</td>
                        <td>{{Botnet ALERT_COUNT}}</td>
                    </tr>
                                    <tr>
                        <td>Leak Alerts</td>
                        <td>{{Leak ALERT_COUNT}}</td>
                    </tr>
                                    <tr>
                        <td>Account Alerts</td>
                        <td>{{Account ALERT_COUNT}}</td>
                    </tr>
                                    <tr>
                        <td>Ransomware Alerts</td>
                        <td>{{Ransomware ALERT_COUNT}}</td>
                    </tr>
                                    <tr>
                        <td>Credit Card Alerts</td>
                        <td>{{Credit Card ALERT_COUNT}}</td>
                </table>
            </div>
            <div class="email-footer">
                This is a {{COMPANY_NAME}} generated alert. Please see the details in the attached file. Please consult with {{COMPANY_EMAIL}} if you have any questions or need assistance in processing this alert.
                <br> <br>
                Your {{COMPANY_NAME}} Team
            </div>
        </div>
    </body>
    </html>
    """
    html = html.replace('{{COMPANY_NAME}}', company_name).replace('{{COMPANY_EMAIL}}', company_contact).replace('{{ALERT_COUNT}}', str(alert_count))
    html = html.replace('{{SSL_ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['certificates']))
    html = html.replace('{{Botnet ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['botnets']))
    html = html.replace('{{Leak ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['leaks']))
    html = html.replace('{{Account ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['accounts']))
    html = html.replace('{{Ransomware ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['ransomware']))
    html = html.replace('{{Credit Card ALERT_COUNT}}', str(RESULTS_FOUND_PER_QUERY['cc']))
    username = sender_email
    password = sender_password
    from_email = sender_email
    to_email = alert_email
    subject = 'DARK & DEEP WEB ALERT NOTIFICATION'
    content = html
    # msg = MIMEMultipart()
    msg = EmailMessage()
    msg['Subject'] = subject
    msg['From'] = from_email
    msg['To'] = to_email
    #msg['To'] = "team34723@gmail.com"
    msg.set_content(html, subtype='html')

    if attachments:
        for attachment in attachments:
            with open(attachment, 'rb') as f:
                file_data = f.read()
                file_name = os.path.basename(f.name)
                msg.add_attachment(file_data, maintype='text', subtype='csv', filename=file_name)


    # msg.attach(MIMEText(content, 'html'))

    try:
        server = smtplib.SMTP(smtp_server, smtp_port)
        server.starttls()
        server.login(username, password)
        server.send_message(msg)
        server.quit()
        print('Email sent successfully')
    except Exception as e:
        print(f'Error while sending email: {e}')


def process_domain(domain):
    # Extract the 2nd level domain (example) using a regex pattern
    match = re.match(r'^(?:.*\.)?([^.]+)\.$', domain)

    if match:
        second_level_domain = match.group(1)

        # Check the length of the 2nd level domain
        if len(second_level_domain) < 5:
            # Query without wildcards
            print(f"Query without wildcards for domain: {second_level_domain}.com")
        else:
            # Query with wildcards
            print(f"Query with wildcards for domain: *.{second_level_domain}.com")
    else:
        print("Invalid domain format")


def main():
    input_method = input("Enter 'manual' to input data manually or 'csv' to load from a CSV file: ")
    data_array = get_input_from_csv() if input_method == 'csv' else get_manual_input()

    # create a json file from data_array[0]
    # add spaces to json file to look pretty
    import json
    with open('data_array.json', 'w') as f:
        json.dump(data_array[0], f, indent=4)

    for data in data_array:
        # Translate '7' into 'all'
        if data['query_type'] == '7':
            data['query_type'] = 'all'

        if data['alert'] == '1':
            # change the daterange '2022-01-01 TO 2023-11-12' to today's date
            today = datetime.datetime.today().strftime('%Y-%m-%d')
            data['date_range'] = f"createdAt:[{today} TO {today}]"

        if data['alert'] not in ["0", "1"]:
            raise ValueError("Invalid alert option. Please enter '0' or '1'.")

        if str(data['smtp_port']) != "587" and str(data['alert']) == "1":
            input("WARNING! SMTP port is not 587. Are you sure you are using encryption? Press Enter to ignore and continue.")

        print(data)

        # Validate input and treat 'asset_name_domain' as a list
        # if not all(is_valid_domain_or_ip(asset_name_domain, data['asset_option']) for asset_name_domain in data['asset_name_domain']):
        #     raise Exception("Invalid field `asset_name_domain`. Please enter a valid domain, 2nd-level domain, or IP range.")

        # Authenticate and get token
        print("Authenticating...")
        token = get_access_token(data)
        print("Authentication successful.")

        # Determine the query type(s) to perform
        if data['query_type'].lower() == 'all':
            query_types = list(API_ENDPOINTS.keys())  # Get all the endpoints if 'all' is selected
        else:
            # Translate query_type using QUERY_OPTIONS
            query_types = [QUERY_OPTIONS[data['query_type']]]

        # print('breakpoint code stopped')
        # breakpoint()


        # Perform queries and treat 'asset_name_domain' as a list
        results = []
        files_to_attach_path = []
        for query_type in query_types:
            if query_type != 'leaks':
                continue
            results_per_query = []
            print(f"Querying {query_type}...")
            endpoint = API_ENDPOINTS[query_type]

            if query_type == 'leaks':
                for asset_name_other in data['asset_name_other']:
                    print(f"DO: asset_name_other value - {asset_name_other}")
                    print(f"Type: {type(asset_name_other)}")
                    if isinstance(asset_name_other, str):
                        print(1)
                        pass
                    # January 29: perform leak query only
                    response = perform_query_leaks(
                        endpoint=endpoint, token=token, query=asset_name_other,
                        query_date_from=data['query_date_from'], query_date_to=data['query_date_to']
                    )
                    RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                    results.extend(response)
                    results_per_query.extend(response)
                    print(f"OK: response len {len(response)}")
                    print("*" * 100)

            for asset_name_domain in data['asset_name_domain']:

                if query_type == 'domain':
                    # Extract the domain from the asset_name_domain
                    if "http://" not in asset_name_domain or "https://" not in asset_name_domain:
                        domain_query = f"https://{asset_name_domain}"
                        domain_query = tldextract.extract(domain_query).domain
                        if len(domain_query) < 5:
                            domain_query = asset_name_domain
                        elif len(domain_query) > 5:
                            domain_query = f"*{domain_query}*"
                        response = perform_query(endpoint, token, query_type, domain_query)
                        RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                        results.extend(response)
                        results_per_query.extend(response)

                if query_type == 'cc':
                    # here needs to query two fields: credit_card_number and credit_card_owner
                    # before querying, let's verify the input fields are not empty
                    query_list_cc_number = []
                    query_list_cc_owner = []
                    if data['credit_card_number']:
                        query_list_cc_number.extend(data['credit_card_number'])
                    if data['credit_card_owner']:
                        query_list_cc_owner.extend(data['credit_card_owner'])

                    # now let's query CC numbers
                    for query_cc in query_list_cc_number:
                        if query_cc != "" and query_cc != " ":
                            response = perform_query(
                                endpoint=endpoint, token=token, query_type=query_type,
                                asset_name=f"{query_cc} AND {data['date_range']}"
                            )
                            RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                            results.extend(response)
                            results_per_query.extend(response)

                    # now let's query CC owners
                    for query_cc in query_list_cc_owner:
                        if query_cc != "" and query_cc != " ":
                            response = perform_query(
                                endpoint=endpoint, token=token, query_type=query_type,
                                asset_name=f"owner:{query_cc} AND {data['date_range']}"
                            )
                            RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                            results.extend(response)
                            results_per_query.extend(response)
                elif query_type == 'ransomware':
                    # here needs to split 'ransomware_query' into a string list
                    # no need to use 'date_range' here
                    query_list_ransomware = data['ransomware_query'].split()
                    for query_ransomware in query_list_ransomware:
                        response = perform_query(endpoint, token, query_type, query_ransomware)
                        RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                        results.extend(response)
                        results_per_query.extend(response)

                    # query supply chains with the same rules as client name
                    query_list_supply_chain_ransomware = ast.literal_eval(data['supply_chain'])
                    for query_supply_chain_ransomware in query_list_supply_chain_ransomware:
                        response = perform_query(endpoint, token, query_type, query_supply_chain_ransomware)
                        RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                        results.extend(response)
                        results_per_query.extend(response)

                elif query_type == 'leaks':
                    # here there could be > 10000 results, so we need to query them separately by chunks and update daterange once the 10K chunk is reached
                    response = perform_query_leaks(
                        endpoint=endpoint, token=token, query=asset_name_domain,
                        query_date_from=data['query_date_from'], query_date_to=data['query_date_to']
                    )
                    RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                    results.extend(response)
                    results_per_query.extend(response)
                else:
                    # for all other query types, just query the asset name
                    response = perform_query(endpoint, token, query_type, asset_name_domain + f" AND {data['date_range']}")
                    RESULTS_FOUND_PER_QUERY[query_type] += len(response)
                    results.extend(response)
                    results_per_query.extend(response)
            if results_per_query:
                saved_file_path = save_to_csv(
                    client_name=data['client_name'],
                    query_type=f"Client_{data['client_name']}__QueryType_{query_type}___Date_{datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}",
                    results=results_per_query
                )
                files_to_attach_path.append(saved_file_path)

        # here comes the alerting part
        if results:
            print(f"Query successful. Retrieved {len(results)} records.")
            save_to_csv(
                client_name=data['client_name'],
                query_type=f"Client_{data['client_name']}__QueryType_{data['query_type']}___Date_{datetime.datetime.now().strftime('%Y-%m-%d-%H-%M-%S')}",
                results=results
            )
            if str(data['alert']) == "1":
                send_email(
                    alert_email=data['alert_email'],
                    sender_email=data['sender_email'],
                    sender_password=data['sender_password'],
                    company_name=data['company_name'],
                    company_contact=data['company_contact'],
                    alert_count=len(results),
                    smtp_server=data['smtp_server'],
                    smtp_port=data['smtp_port'],
                    attachments=None
                )
            if str(data['attach_results_to_email']) == "1":
                send_email(
                    alert_email=data['alert_email'],
                    sender_email=data['sender_email'],
                    sender_password=data['sender_password'],
                    company_name=data['company_name'],
                    company_contact=data['company_contact'],
                    alert_count=len(results),
                    smtp_server=data['smtp_server'],
                    smtp_port=data['smtp_port'],
                    attachments=files_to_attach_path
                )
        else:
            print(f"No results found for query_type {data['query_type']}.")

if __name__ == "__main__":
    main()
